@@ -2,7 +2,10 @@ mod file_list;
 mod player_state;
 mod user_input;
 
-use crate::*;
+use crate::{
+  mpris::{build_metadata, PlaybackStatus},
+  *,
+};
 use anyhow::Result;
 use crossbeam_channel::{Receiver, Sender};
 use crossterm::{
@@ -33,6 +36,11 @@ pub enum Focusable {
 pub enum AppMessage {
   Select(usize),
   SelectDelta(i64),
+  Play(Option<PathBuf>),
+  Pause,
+  PlayPause,
+  Next,
+  Prev,
 }
 
 pub struct App {
@@ -46,7 +54,11 @@ pub struct App {
   pub progress: (f64, u64, u64),
   pub playing: Option<Arc<Node>>,
   pub play_index: usize,
-  mailbox: (Sender<AppMessage>, Receiver<AppMessage>),
+  mailbox: (Arc<Sender<AppMessage>>, Receiver<AppMessage>),
+  last_played: Option<Arc<Node>>,
+
+  #[cfg(feature = "mpris")]
+  mpris_tx: Sender<PlaybackStatus>,
 }
 
 impl App {
@@ -56,7 +68,20 @@ impl App {
 
     let progress = backend.progress();
 
-    let mailbox = crossbeam_channel::unbounded();
+    let (sender, receiver) = crossbeam_channel::unbounded();
+    let sender = Arc::new(sender);
+
+    #[cfg(feature = "mpris")]
+    let mpris_tx = {
+      let (mpris_tx, mpris_rx) = crossbeam_channel::unbounded();
+      let mailbox = sender.clone();
+
+      std::thread::spawn(move || {
+        let _ = mpris::run_dbus_server(mailbox, mpris_rx);
+      });
+
+      mpris_tx
+    };
 
     let mut app = Self {
       backend,
@@ -69,7 +94,11 @@ impl App {
       window_offset: 0,
       progress,
       play_index: 0,
-      mailbox,
+      mailbox: (sender, receiver),
+      last_played: None,
+
+      #[cfg(feature = "mpris")]
+      mpris_tx,
     };
     // app.set_root(&path);
 
@@ -137,6 +166,8 @@ impl App {
     key: &KeyEvent,
     terminal: &mut Terminal<B>,
   ) -> Result<()> {
+    use AppMessage::*;
+
     let ctrl = key.modifiers.contains(KeyModifiers::CONTROL);
     let alt = key.modifiers.contains(KeyModifiers::ALT);
 
@@ -151,16 +182,16 @@ impl App {
         std::process::exit(0);
       }
       (KeyCode::Down, _, _) | (KeyCode::Char('n'), true, _) => {
-        self.message(AppMessage::SelectDelta(1));
+        self.message(SelectDelta(1));
       }
       (KeyCode::Up, _, _) | (KeyCode::Char('p'), true, _) => {
-        self.message(AppMessage::SelectDelta(-1));
+        self.message(SelectDelta(-1));
       }
       (KeyCode::Char('v'), true, _) => {
-        self.message(AppMessage::SelectDelta(10));
+        self.message(SelectDelta(10));
       }
       (KeyCode::Char('v'), _, true) => {
-        self.message(AppMessage::SelectDelta(-10));
+        self.message(SelectDelta(-10));
       }
       _ => match self.focus {
         Focusable::FileList => file_list::handle_input(self, key),
@@ -210,6 +241,8 @@ impl App {
   }
 
   fn update(&mut self, list_state: &mut ListState) -> Result<()> {
+    use AppMessage::*;
+
     self.progress = self.backend.progress();
 
     self.ensure_continue();
@@ -217,13 +250,18 @@ impl App {
     let messages: Vec<AppMessage> = self.mailbox.1.try_iter().collect();
     for msg in messages {
       match msg {
-        AppMessage::Select(index) => {
+        Select(index) => {
           self.select(index, list_state);
         }
-        AppMessage::SelectDelta(delta) => {
+        SelectDelta(delta) => {
           let index = self.selected.unwrap_or(0) as i64 + delta;
           self.select(index.max(0) as usize, list_state);
         }
+        Play(_path) => self.backend.play(None),
+        PlayPause => self.backend.play_pause(),
+        Pause => self.backend.pause(),
+        Next => self.play(self.play_index + 1),
+        Prev => self.play(self.play_index.saturating_sub(1)),
       }
     }
 
@@ -244,14 +282,23 @@ impl App {
     match self.library.file_list().get(index) {
       Some((node, _)) => {
         if node.is_file() {
-          self.backend.play(&node.path);
+          #[cfg(feature = "mpris")]
+          {
+            let _ = self
+              .mpris_tx
+              .send(PlaybackStatus::Playing(Some(build_metadata(node))));
+          }
+
+          self.last_played = Some(node.clone());
+          self.backend.play(Some(&node.path));
           return;
         }
+
         self.expand(index);
         self.play(index + 1);
       }
       None => {
-        self.backend.pause();
+        self.pause();
       }
     }
     self.message(AppMessage::Select(index));
@@ -287,9 +334,28 @@ impl App {
     }
   }
 
+  pub fn pause(&mut self) {
+    self.backend.pause();
+
+    #[cfg(feature = "mpris")]
+    let _ = self.mpris_tx.send(PlaybackStatus::Paused);
+  }
+
+  pub fn play_pause(&mut self) {
+    self.backend.play_pause();
+
+    #[cfg(feature = "mpris")]
+    let _ = match self.backend.is_paused() {
+      true => self.mpris_tx.send(PlaybackStatus::Paused),
+      false => self.mpris_tx.send(PlaybackStatus::Playing(
+        self.last_played.as_ref().map(|lp| build_metadata(&lp)),
+      )),
+    };
+  }
+
   fn select(&mut self, index: usize, list_state: &mut ListState) {
     let height = self.height as usize - 1;
-    let index = index.min(self.library.file_list().len() - 1);
+    let index = index.min(self.library.file_list().len());
 
     self.selected = Some(index);
     self.window_offset = self
